#include <algorithm>
#include <iostream>
#include <ranges>

#include "AtmAdmin.h"
#include "Errorseal.h"

namespace AtmAdmin {
    char startup() {
        char choice{};
        std::cout << "1. Change PIN\n" << "2. Change account name\n" << "3. Change money\n" << "Q. Exit the program\n\n";
        std::cout << ">: ";

        std::cin >> choice;

        return choice;
    }

    // Controls the flow of the program based on the user's input.
    bool handleUserInformation(char choice, std::string& information) {
        std::string tempInformation{};

        switch (choice) {
            case '1':
                tempInformation = changeAccountPin();
                if (tempInformation != "invalid") {
                    information = tempInformation;
                    return true;
                }
                break;
            case '2':
                // Implement a function for entering a new username.
                // The function should check if the name isn't already taken. If it is, return to start.
                // It will also use an algorithm to check if it's a valid username
                // This includes no spaces and no starting numbers.
                // Ensure that the name is also converted to fully lowercase.

                // The code below is a placeholder
                tempInformation = changeAccountName();
                if (tempInformation != "invalid") {
                    information = tempInformation;
                    return true;
                }
                break;
            case '3':
                // Simply a way to cheat, very straight-forward.

                // The code below is a placeholder
                std::cout << "Enter the amount you wish to have: ";
                std::cin >> information;
                break;
            case 'q':
                return false;
            default:
                std::cout << "Invalid choice.\n";
        }
        return true;
    }

    // Allows for the quick and clean creation of error reports.
    void errorReport (std::string_view error) {
        std::cout << error << '\n';
        std::cout << "Press enter to continue...";
        std::cin.get();
    };

    std::string changeAccountPin() {
        std::string newPin{};

        std::cout << "Enter new pin: ";
        std::cin >> newPin;
        Error::ignoreLine();

        std::array<int, 4> pinDigits{};

        for (size_t i{}; i != 4; i++) {
            pinDigits[i] = (newPin[i] - '0');
        }

        if (newPin.size() != 4) {
            errorReport("The pin has to be 4 numbers long.");
            return "invalid";
        }

        if (newPin[0] == '0') {
            errorReport("The first number cannot be \'0\'");
            return "invalid";
        }

        // Checks if anything put into the pin is alphabetic.
        for (size_t i = 0; i < 4; i++) {
            if (isalpha(newPin[i])) {
                errorReport("The pin cannot contain alphabet letters.");
                return "invalid";
            }
        }

        // Checks for sequential numbers (examples: 1234, 6789)
        { // START OF BRACKET SCOPE //
            int sequentialCount {0};

            for (size_t i{0}; i < 3; i++) {
                if (pinDigits[i]+1 == pinDigits[i+1] or pinDigits[i]-1 == pinDigits[i+1])
                    sequentialCount += 1;


                if (sequentialCount == 3) {
                    errorReport("Pin is too simple.");
                    return "invalid";
                }
            }
        } // END OF BRACKET SCOPE //

        // Checks for too many duplicate numbers.
        { // START OF BRACKET SCOPE //
            std::string tempPin = newPin;
            std::ranges::sort(tempPin.begin(), tempPin.end());


            if ( (tempPin[0] == tempPin[1] and tempPin[1] == tempPin[2]) // Checks if the first 3 numbers are the same.
                 or
                 (tempPin[1] == tempPin[2] and tempPin[2] == tempPin[3]) // Checks if the last 3 numbers are the same.
                 or
                 (tempPin[0] == tempPin[1] and tempPin[2] == tempPin[3])) // Checks if the first 2 and last 2 numbers are the same.
            {
                errorReport("Too many duplicate numbers.");
                return "invalid";
            }
        } // END OF BRACKET SCOPE //

       return newPin;
    }

    std::string changeAccountName() {
        std::string newName{};


        std::cout << "Enter new name: ";
        std::cin >> newName;
        Error::ignoreLine();

        // Ensures the name is between 2 and 10 characters long.
        if (newName.length() < 2 or newName.length() > 10) {
            errorReport("Name has to be between 2 and 10 characters.");
            return "invalid";
        }

        // Checks if the first character is a number.
        if (isdigit(newName.at(0))) {
            errorReport("First letter cannot be a digit.");
            return "invalid";
        }

        // Turns all the letters into lowercase.
        std::ranges::transform (newName.cbegin(), newName.cend(), newName.begin(), [] (unsigned char input) { return tolower(input); });
        return newName;
    }
}